#ifndef _VEBUS_DEV_UPDATE_H_
#define _VEBUS_DEV_UPDATE_H_

#include <velib/mk2/mk2.h>
#include <velib/vebusupd/vff_file.h>

typedef veBool VbdupUpdateBusy(void);
typedef void VbdupFirmwareUpdateTick(void);
typedef void VbdupUpdateRxByte(un8 inbyte);


/* Bootblock version
 * 15..6 = bootblock type
 * 5..0  = revision
 *   bootblock type is subdivided in families and single units
 *	for instance:
 *	  0b00000000xxxx:	(Multi,Panel,SWL etc which can all use the same bootblock source but have small adaptations
 *					for the error loops and TX hardware.fundamental differences (MK2<-> Multi,Panel,SWL)
 *	  0b000000010000:	MK3
 */
typedef enum {
	MULTI_BOOTBLOCK = 0, /* 0x00? is for Multis */
	MK3_BOOTBLOCK = 0x10
} BootBlockType;


typedef enum {
VBDUP_OK,
VBDUP_UPDATE_FAILED,
} VbdupUpdateResult;

typedef void VbdubUpdateResultCallback(VbdupUpdateResult result);

typedef enum {
	CT_UNKNOWN,
	CT_MK2,
	CT_MK3,
	CT_PIC18K_NEW,
	CT_SWL,
	CT_OLD_FASHIONED,
} VbdupConnectType;

typedef enum {
	CM_UNKNOWN,
	CM_ACTIVE,
	CM_PASSIVE,
} VbdupConnectMode;

typedef struct
{
	VbdupConnectType ct;
	VbdupConnectMode cm;
	VbdupUpdateBusy* pVbdupUpdateBusy;
	VbdupFirmwareUpdateTick* pVbdupFirmwareUpdateTick;
	VbdupUpdateRxByte* pVbdupUpdateRxByte;
	VffData* pVffData;
	int cpIndex;
} VbdupConnectInfo;

veBool vbdupUpdate (VbdupConnectInfo *pCi);
veBool vbdupUpdateBootblock (VbdupConnectInfo *pCi);
veBool vbdupUpdateBusy(VbdupConnectInfo *pCi);
void vbdubUpdateTick(VbdupConnectInfo *pCi);
void vebusDeviceRxUpdateRxByte(un8 inbyte); /* Incoming data should be passed here while updating */


#endif	/* _VEBUS_DEV_UPDATE_H_ */
