#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <velib/base/base.h>
#include <velib/base/ve_string.h>
#include <velib/platform/console.h>
#include <velib/types/ve_str.h>
#include <velib/utils/ve_arg.h>

#include "mk3selftest.h"


int skipHwLineTests = veFalse;
int setStandbyLine = veFalse;
int setPanelDetectLine = veFalse;
int skipUpdate = veFalse;

struct option consoleOptions[] =
{
	{"skip-line-tests",	no_argument,		&skipHwLineTests,			1},
	{"set-standby-line",	no_argument,		&setStandbyLine,			1},
	{"set-panel-line",	no_argument,		&setPanelDetectLine,		1},
	{"tester-mode",		no_argument,		0,							't'},
	{"skip-update",		no_argument,		&skipUpdate,				1}	,
	{0,					0,					0,							0}
};

void consoleUsage(char* program)
{
	printf("%s [Victron MK3 Selftest] version %s\n", program, MK3SELFTEST_VERSION);
	printf("\n");

	printf("The device under test (DUT) MK3 should be connected to a tester MK3.\n");
	printf("Power to the VE.Bus cable should be supplied by either a Multi in Freeze mode.\n");
	printf("(No. VE.Bus communication) or a sepparate tester box.\n");

#ifdef WINDOWS
	printf("NOTE: In order to access com port numbers > 9 use the following format:\n");
	printf("-s \\\\.\\COM##\n");
#endif

	printf("\n");
	printf("OPTIONS:\n");

	printf("\n");
	printf("  --skip-line-tests\n");
	printf("    Don't run mk3 stanby and paneldetect line tests.\n");
	printf("\n");

	printf("\n");
	printf("  --skip-update\n");
	printf("    Don't update MK3 firmware if required.\n");
	printf("\n");

	printf("  --set-standby-line\n");
	printf("    Set the standby line (DUT mode only).\n");
	printf("\n");

	printf("  --set-panel-line\n");
	printf("    Set the panel detect line (DUT mode only).\n");
	printf("\n");

	printf("  --tester-mode\n");
	printf("    Run in tester mode. Default mode is DUT mode.\n");
	printf("\n");


	pltOptionsUsage();
	printf("Victron Energy B.V.\n");
}

veBool consoleOption(int flag)
{
	switch (flag)
	{
		case 0:
			break;

		case 't':
			mk3SelftestSetTesterMode(veTrue);
			break;

		default:
			fprintf(stderr, "invalid option\n");
			return veFalse;
	}

	return veTrue;
}

veBool consoleArgs(int argc, char *argv[])
{
	VE_UNUSED(argv);

	if (argc != 0)
	{
		printf("error - no arguments are expected - missing a '-' or '--'?");
		return veFalse;
	}

	return veTrue;
}

#if 0
char const *pltProgramVersion(void)
{
	return VERSION_STRING;
}
#endif
