/** This is the main mk3 device updater */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <velib/velib_inc_mk2.h>
#include <velib/base/base.h>
#include <velib/base/endian.h>

#include <velib/platform/plt.h>
#include <velib/platform/task.h>
#include <velib/utils/ve_logger.h>
#include <velib/base/ve_string.h>
#include <velib/mk2/frame_handler.h>
#include <velib/mk2/mk2.h>
#include <velib/mk2/hal.h>
#include <velib/mk2/mk2_actions.h>
#include <velib/mk2/mk2_strings.h>
#include <velib/mk2/mk2_update.h>
#include <velib/mk2/vebus_system.h>
#include <velib/mk2/vebus_device.h>
#include <velib/mk2/vebus_items.h>
#include <velib/mk2/values_float.h>
#include <velib/vebusupd/vff_file.h>
#include <velib/vebusupd/vebus_device_update.h>
#include <velib/mk3/mk3.h>

#include <../../inc/mk3_firmware_upgrade.h> /* Use the same firmware version as mk2dbus */


static Mk2ResultCallback *mk3UpdateDoneCb;
static VffFile vffFile;
static VffData dataBuf;
static VbdupConnectInfo vbdupCi;
static veBool upgradeBusy = veFalse;

static void mk3FirmwareUpradeExit(int err)
{
	if (vffFile.pFile != NULL)
		vffFileClose(&vffFile);
	if(dataBuf.pStart != NULL)
		vffFreeDataBuffer(&dataBuf);

	if(err != 0)
		logI("mk3FirmwareUpgrade","Exit code = %d", err);

	logI("mk3FirmwareUpgrade","Done");

	upgradeBusy = veFalse;

	if (mk3UpdateDoneCb) {
		mk3UpdateDoneCb(err);
		mk3UpdateDoneCb = NULL;
	}

}


void mk3FirmwareUpgrade(Mk2ResultCallback *mk3UpdateDone)
{
	int result, size;

	mk3UpdateDoneCb = mk3UpdateDone;

	upgradeBusy = veTrue;

	result = vffFileOpen(&vffFile, MK3_FIRMWARE_FILE_NAME);

	if (result != 0)
		goto onError;

	size = vffFile.endLocation;
	logI("mk3FirmwareUpgrade", "Malloc vff line buffer with size %d", size);

	result = vffMallocDataBuffer(&dataBuf, size);
	if (result != 0)
		goto onError;

	result = vffParseFile(&vffFile, &dataBuf);
	if (result != 0)
		goto onError;

	vbdupCi.pVffData = &dataBuf;

	vbdupUpdate(&vbdupCi);

	return;

onError:
	mk3FirmwareUpradeExit(result);
	return;
}


void mk3FirmwareUpgradeTick (void)
{
	if (vbdupUpdateBusy(&vbdupCi)) {
		vbdubUpdateTick(&vbdupCi);
	}
	else {
		logI("mk3FirmwareUpgrade", "Update done");
		mk3FirmwareUpradeExit(0);
	}
}


veBool mk3FirmwareUpgradeBusy (void)
{
	return upgradeBusy;
}
